// This is a utility program to process existing MFM delta transition data.
// Used to extract the sector contents to a file
//
// deltas_get_count routine is used by called MFM decoding routines
//
// Copyright 2013 David Gesswein.
// This file is part of MFM disk utilities.
//
// MFM disk utilities is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// MFM disk utilities is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with MFM disk utilities.  If not, see <http://www.gnu.org/licenses/>.
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include <stdint.h>
#include <errno.h>

#include "msg.h"
#include "crc_ecc.h"
#define DEF_DATA
#include "mfm_decoder.h"
#include "parse_cmdline.h"
// We don't use the routines from deltas_read.h since we aren't using beaglebone PRU
// we include it to make sure our versions at the end of the file match the
// parameters the other routines are using.
#include "deltas_read.h"
#include "emu_tran_file.h"

#define MAX_DELTAS 131072
// The number of deltas we have to process
uint32_t num_deltas;

static void deltas_update_count(int num_deltas_in);


// Main routine
int main (int argc, char *argv[])
{
   uint16_t deltas[MAX_DELTAS];
   int num_deltas;
   int cyl, head;
   int last_cyl = -1, last_head = -1;
   DRIVE_PARAMS drive_params;
   SECTOR_STATUS sector_status_list[MAX_SECTORS];
   int seek_difference;
   SECTOR_DECODE_STATUS status;
   int analyze;
   int transition_file = 1;
   EMU_FILE_INFO emu_file_info;
   TRAN_FILE_INFO tran_file_info;

   // Get what to do. Turn off options we don't support
   parse_cmdline(argc, argv, &drive_params, &analyze, "arj");

   if (drive_params.transitions_filename == NULL) {
      if (drive_params.emulation_filename == NULL) {
         msg(MSG_FATAL, "Transition or emulation filename must be specified\n");
         exit(1);
      } else {
         transition_file = 0;
         // Fix up from parse_cmdline. We read emulation file not write it if no
         // transition file is specified
         drive_params.emulation_output = 0;
         emu_file_read_header(drive_params.emulation_filename, &emu_file_info, 0);
      }
   } else {
      tran_file_read_header(drive_params.transitions_filename, &tran_file_info);
   }

   // Setup decoding of transitions and possible file to write to
   mfm_decode_setup(&drive_params, 1);

   if (transition_file) {
      num_deltas = tran_file_read_track_deltas(deltas, MAX_DELTAS, &cyl, &head);
   } else {
      num_deltas = emu_file_read_track_deltas(deltas, MAX_DELTAS, &cyl, &head);
   }
   // Read and process a track at a time until all read
   while (num_deltas >= 0) {
      // Only clear status if we are moving to the next track. If retries
      // were done we may have multiple reads of the same track.
      if (last_cyl != cyl || last_head != head) {
         mfm_init_sector_status_list(sector_status_list, drive_params.num_sectors);
      }
      //printf("Decoding new track %d %d\n",cyl, head);
      deltas_update_count(num_deltas);
      // If head & cylinder haven't changed assume it's a retry.
      status = mfm_decode_track(&drive_params, cyl, head, deltas, &seek_difference,
            sector_status_list);
#if 0
      if (1 || status != SECT_HEADER_FOUND) {
         if (cyl == 0 && head == 0) {
            FILE *file;
            char fn[256];
            int i;
            sprintf(fn, "trk%d-%d.txt",cyl, head);
            file = fopen(fn, "w");
            for (i = 0; i < num_deltas; i++) {
               fprintf(file, "%d\n",deltas[i]);
            }
            fclose(file);
         }
      }
#endif
      last_cyl = cyl;
      last_head = head;
      if (transition_file) {
         num_deltas = tran_file_read_track_deltas(deltas, MAX_DELTAS, &cyl, &head);
      } else {
         num_deltas = emu_file_read_track_deltas(deltas, MAX_DELTAS, &cyl, &head);
      }
   }
   mfm_decode_done(&drive_params);
   return 0;
}

// Update the count of deltas.
//
// num_deltas_in: Number of deltas read
static void deltas_update_count(int num_deltas_in)
{
   num_deltas = num_deltas_in;
}

// This routines replace the routines from delta=_read that are used
// when reading from a real drive.

// Return deltas available or -1 if caller has processed all the deltas.
//
// cur_delta: Number of deltas processed so far
// return: Number of deltas available or -1 if all processed.
int deltas_get_count(int deltas_processed)
{
   if (deltas_processed >= num_deltas) {
      return -1;
   } else {
      return num_deltas;
   }
}
